/**
 * Copyright (c) 2005 by De Wolf Consultancy
 * All Rights Reserved.
 * 
 * $Id: JaasSecurityManagerService.java,v 1.2 2005/12/14 21:22:22 carlo Exp $
 */
package nl.wolfc.embedded.security.plugins;

import java.io.IOException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.Name;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.spi.ObjectFactory;
import javax.security.auth.login.Configuration;
import javax.security.auth.login.LoginException;

import nl.wolfc.naming.BrainlessContext;

import org.apache.log4j.Logger;
import org.jboss.security.auth.callback.SecurityAssociationHandler;
import org.jboss.security.auth.login.XMLLoginConfigImpl;

/**
 * @author carlo
 *
 */
public class JaasSecurityManagerService {
	@SuppressWarnings("unused")
	private static final Logger log = Logger.getLogger(JaasSecurityManager.class);
	
	private static final String SECURITY_MGR_PATH = "java:/jaas";
	
	private static Map<String, JaasSecurityManager> cache = new HashMap<String, JaasSecurityManager>();
	
	public JaasSecurityManagerService() throws NamingException, IOException, LoginException {
		XMLLoginConfigImpl configuration = new XMLLoginConfigImpl();
		configuration.setConfigResource("login-config.xml");
		configuration.loadConfig();
		
		Configuration.setConfiguration(configuration);
		
		Context ctx = new InitialContext();
		
//		ctx = (Context) ctx.lookup("java:");
//		ctx = ctx.createSubcontext("jaas");
//		//ctx.bind("other", new MyAuthenticationManager("other"));
//		NonSerializableFactory.rebind(ctx, "other", new JaasSecurityManager("other", new SecurityAssociationHandler()));
//		//ctx.bind("other", new JaasSecurityManager("other", new SecurityAssociationHandler()));
		
//		RefAddr refAddr = new StringRefAddr("nns", "JSM");  
//		String factoryName = SecurityDomainObjectFactory.class.getName();  
//		Reference ref = new Reference("javax.naming.Context", refAddr, factoryName, null);  
//		ctx.rebind(SECURITY_MGR_PATH, ref);
		
		//RefAddr refAddr = new StringRefAddr("nns", "JSM");  
		String factoryName = SecurityDomainObjectFactory.class.getName();  
		Reference ref = new Reference("nl.wolfc.embedded.security.plugins.JaasSecurityManager", factoryName, null);  
		ctx.rebind(SECURITY_MGR_PATH, ref);  
	}
	
	private static JaasSecurityManager getSecurityManager(String name) {
		JaasSecurityManager manager = cache.get(name);
		if(manager != null) {
			//log.info("cache hit");
			return manager;
		}
		synchronized(cache) {
			if(manager != null)
				return manager;
			
			manager = new JaasSecurityManager(name, new SecurityAssociationHandler());
			cache.put(name, manager);
		}
		return manager;
	}
	
	public static class SecurityDomainObjectFactory implements ObjectFactory {
		public Object getObjectInstance(Object obj, Name name, Context nameCtx, Hashtable<?, ?> environment) throws Exception {
			return new BrainlessContext() {
				public Object lookup(Name name) throws NamingException {
					return lookup(name.get(0));
				}
				
				public Object lookup(String name) throws NamingException {
					return getSecurityManager(name);
				}
			};
		}
	}
}
